@@ -1,9 +1,9 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 # -*- coding: utf-8 -*-
 #
 # SoundConverter - GNOME application for converting between audio formats.
 # Copyright 2004 Lars Wirzenius
-# Copyright 2005-2014 Gautier Portet
+# Copyright 2005-2017 Gautier Portet
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,39 +21,37 @@
 
 import os
 import sys
-from urlparse import urlparse
+from urllib.parse import urlparse
 from gettext import gettext as _
 
-import gobject
-import gst
-import gst.pbutils
-import gnomevfs
-import gconf
+import gi
+from gi.repository import Gst, Gtk, GObject, Gio
+
+from soundconverter.fileoperations import vfs_encode_filename, file_encode_filename
+from soundconverter.fileoperations import unquote_filename, vfs_makedirs, vfs_unlink
+from soundconverter.fileoperations import vfs_rename
+from soundconverter.fileoperations import vfs_exists
+from soundconverter.fileoperations import beautify_uri
+from soundconverter.task import BackgroundTask
+from soundconverter.queue import TaskQueue
+from soundconverter.utils import debug, log, idle
+from soundconverter.settings import mime_whitelist, filename_blacklist
+from soundconverter.error import show_error
 
-from fileoperations import vfs_encode_filename, file_encode_filename
-from fileoperations import unquote_filename, vfs_makedirs, vfs_unlink
-from fileoperations import vfs_rename
-from fileoperations import vfs_exists
-from fileoperations import beautify_uri
-from fileoperations import use_gnomevfs
-from task import BackgroundTask
-from queue import TaskQueue
-from utils import debug, log
-from settings import mime_whitelist, filename_blacklist
-from error import show_error
 try:
-    from notify import notification
+    from soundconverter.notify import notification
 except:
     def notification(msg):
         pass
-        
+
 from fnmatch import fnmatch
 
 import time
-import gtk
+
+
 def gtk_iteration():
-    while gtk.events_pending():
-        gtk.main_iteration(False)
+    while Gtk.events_pending():
+        Gtk.main_iteration(False)
 
 
 def gtk_sleep(duration):
@@ -62,89 +60,78 @@ def gtk_sleep(duration):
         time.sleep(0.010)
         gtk_iteration()
 
-import gconf
 # load gstreamer audio profiles
-_GCONF_PROFILE_PATH = "/system/gstreamer/0.10/audio/profiles/"
-_GCONF_PROFILE_LIST_PATH = "/system/gstreamer/0.10/audio/global/profile_list"
+_GCONF_PROFILE_PATH = "/system/gstreamer/1.0/audio/profiles/"
+_GCONF_PROFILE_LIST_PATH = "/system/gstreamer/1.0/audio/global/profile_list"
 audio_profiles_list = []
 audio_profiles_dict = {}
 
-_GCONF = gconf.client_get_default()
-profiles = _GCONF.get_list(_GCONF_PROFILE_LIST_PATH, 1)
-for name in profiles:
-    if _GCONF.get_bool(_GCONF_PROFILE_PATH + name + "/active"):
-        # get profile
-        description = _GCONF.get_string(_GCONF_PROFILE_PATH + name + "/name")
-        extension = _GCONF.get_string(_GCONF_PROFILE_PATH + name + "/extension")
-        pipeline = _GCONF.get_string(_GCONF_PROFILE_PATH + name + "/pipeline")
-        # check profile validity
-        if not extension or not pipeline:
-            continue
-        if not description:
-            description = extension
-        if description in audio_profiles_dict:
-            continue
-        # store
-        profile = description, extension, pipeline
-        audio_profiles_list.append(profile)
-        audio_profiles_dict[description] = profile
+try:
+    from gi.repository import GConf
+    _GCONF = GConf.Client.get_default()
+    profiles = _GCONF.all_dirs(_GCONF_PROFILE_LIST_PATH)
+    for name in profiles:
+        if _GCONF.get_bool(_GCONF_PROFILE_PATH + name + "/active"):
+            # get profile
+            description = _GCONF.get_string(_GCONF_PROFILE_PATH + name + "/name")
+            extension = _GCONF.get_string(_GCONF_PROFILE_PATH + name + "/extension")
+            pipeline = _GCONF.get_string(_GCONF_PROFILE_PATH + name + "/pipeline")
+            # check profile validity
+            if not extension or not pipeline:
+                continue
+            if not description:
+                description = extension
+            if description in audio_profiles_dict:
+                continue
+                # store
+            profile = description, extension, pipeline
+            audio_profiles_list.append(profile)
+            audio_profiles_dict[description] = profile
+except ImportError:
+    pass
 
 required_elements = ('decodebin', 'fakesink', 'audioconvert', 'typefind', 'audiorate')
 for element in required_elements:
-    if not gst.element_factory_find(element):
-        print("required gstreamer element \'%s\' not found." % element)
+    if not Gst.ElementFactory.find(element):
+        print(("required gstreamer element \'%s\' not found." % element))
         sys.exit(1)
 
-if gst.element_factory_find('giosrc'):
-    gstreamer_source = 'giosrc'
-    gstreamer_sink = 'giosink'
-    encode_filename = vfs_encode_filename
-    use_gnomevfs = True
-    print('  using gio')
-elif gst.element_factory_find('gnomevfssrc'):
-    gstreamer_source = 'gnomevfssrc'
-    gstreamer_sink = 'gnomevfssink'
-    encode_filename = vfs_encode_filename
-    use_gnomevfs = True
-    print('  using deprecated gnomevfssrc')
-else:
-    gstreamer_source = 'filesrc'
-    gstreamer_sink = 'filesink'
-    encode_filename = file_encode_filename
-    print('  not using gnomevfssrc, look for a gnomevfs gstreamer package.')
+gstreamer_source = 'giosrc'
+gstreamer_sink = 'giosink'
+encode_filename = vfs_encode_filename
+print('  using gio')
 
 # used to dismiss codec installation if the user already canceled it
 user_canceled_codec_installation = False
 
 encoders = (
-    ('flacenc',   'FLAC'),
-    ('wavenc',    'WAV'),
+    ('flacenc', 'FLAC'),
+    ('wavenc', 'WAV'),
     ('vorbisenc', 'Ogg Vorbis'),
-    ('oggmux',    'Ogg Vorbis'),
-    ('id3v2mux',  'MP3 Tags'),
-    ('xingmux',   'Xing Header'),
-    ('lame',      'MP3'),
-    ('faac',      'AAC'),
-    ('mp4mux',    'AAC'),
-    ('opusenc',   'Opus'),
-    )
+    ('oggmux', 'Ogg Vorbis'),
+    ('id3mux', 'MP3 tags'),
+    ('xingmux', 'VBR tags'),
+    ('lamemp3enc', 'MP3'),
+    ('faac', 'AAC'),
+    ('mp4mux', 'AAC'),
+    ('opusenc', 'Opus'),
+)
 
 available_elements = set()
 
 for encoder, name in encoders:
-    have_it = bool(gst.element_factory_find(encoder))
+    have_it = bool(Gst.ElementFactory.find(encoder))
     if have_it:
         available_elements.add(encoder)
     else:
-        print ('  "%s" gstreamer element not found'
-            ', disabling %s output.' % (encoder, name))
+        print(('  "%s" gstreamer element not found'
+               ', disabling %s output.' % (encoder, name)))
 
 if 'oggmux' not in available_elements:
     available_elements.discard('vorbisenc')
 
 
 class Pipeline(BackgroundTask):
-
     """A background task for running a GstPipeline."""
 
     def __init__(self):
@@ -186,9 +173,9 @@ class Pipeline(BackgroundTask):
             return
 
         if paused:
-            self.pipeline.set_state(gst.STATE_PAUSED)
+            self.pipeline.set_state(Gst.State.PAUSED)
         else:
-            self.pipeline.set_state(gst.STATE_PLAYING)
+            self.pipeline.set_state(Gst.State.PLAYING)
 
     def found_tag(self, decoder, something, taglist):
         pass
@@ -202,56 +189,63 @@ class Pipeline(BackgroundTask):
         self.play()
 
     def install_plugin_cb(self, result):
-        if result in (gst.pbutils.INSTALL_PLUGINS_SUCCESS,
-                      gst.pbutils.INSTALL_PLUGINS_PARTIAL_SUCCESS):
-            gst.update_registry()
+        return # XXX
+        if result in (Gst.pbutils.INSTALL_PLUGINS_SUCCESS,
+                      Gst.pbutils.INSTALL_PLUGINS_PARTIAL_SUCCESS):
+            Gst.update_registry()
             self.restart()
             return
-        if result == gst.pbutils.INSTALL_PLUGINS_USER_ABORT:
+        if result == Gst.pbutils.INSTALL_PLUGINS_USER_ABORT:
             self.error = _('Plugin installation aborted.')
             global user_canceled_codec_installation
             user_canceled_codec_installation = True
             self.done()
             return
         self.done()
-        show_error('Error', 'failed to install plugin!')
+        show_error('Error', 'failed to install plugins: %s' % GObject.markup_escape_text(str(result)))
 
     def on_error(self, error):
         self.error = error
-        log('error: %s (%s)' % (error, self.command))
+        log('error: %s (%s)' % (error, ' ! '.join(self.command)))
 
+    def on_message_(self, bus, message):
+        self.on_message_(bus, message)
+        return True
+
+    #@idle
     def on_message(self, bus, message):
+        import threading
+
         t = message.type
-        import gst
-        if t == gst.MESSAGE_ERROR:
-            error, _ = message.parse_error()
+        #print('ONMESSAGE', t, threading.currentThread())
+        if t == Gst.MessageType.ERROR:
+            error, __ = message.parse_error()
             self.eos = True
             self.error = error
             self.on_error(error)
             self.done()
-        elif gst.pbutils.is_missing_plugin_message(message):
+            """ XXX elif Gst.pbutils.is_missing_plugin_message(message):
             global user_canceled_codec_installation
-            detail = gst.pbutils.missing_plugin_message_get_installer_detail(message)
-            plugin_name = detail.split('|')[3]
-            debug('missing plugin:', plugin_name, self.sound_file.uri)
-            mime_type = detail.split('|')[4].split(', ')[0]
-            if 'video' in mime_type:
-                debug('  ignoring video codec: ', mime_type)
+            detail = Gst.pbutils.missing_plugin_message_get_installer_detail(message)
+            debug('missing plugin:', detail.split('|')[3] , self.sound_file.uri)
+            self.pipeline.set_state(Gst.State.NULL)
+            if Gst.pbutils.install_plugins_installation_in_progress():
+                while Gst.pbutils.install_plugins_installation_in_progress():
+                    gtk_sleep(0.1)
+                self.restart()
                 return
             if user_canceled_codec_installation:
                 self.error = 'Plugin installation cancelled'
                 debug(self.error)
                 self.done()
                 return
-            self.pipeline.set_state(gst.STATE_NULL)
-            ctx = gst.pbutils.InstallPluginsContext()
-            gst.pbutils.install_plugins_async([detail], ctx, self.install_plugin_cb)
-
-        elif t == gst.MESSAGE_EOS:
+            ctx = Gst.pbutils.InstallPluginsContext()
+            Gst.pbutils.install_plugins_async([detail], ctx, self.install_plugin_cb)
+            """
+        elif t == Gst.MessageType.EOS:
             self.eos = True
             self.done()
-
-        elif t == gst.MESSAGE_TAG:
+        elif t == Gst.MessageType.TAG:
             self.found_tag(self, '', message.parse_tag())
         return True
 
@@ -260,7 +254,7 @@ class Pipeline(BackgroundTask):
             command = ' ! '.join(self.command)
             debug('launching: \'%s\'' % command)
             try:
-                self.pipeline = gst.parse_launch(command)
+                self.pipeline = Gst.parse_launch(command)
                 bus = self.pipeline.get_bus()
                 assert not self.connected_signals
                 self.connected_signals = []
@@ -274,7 +268,7 @@ class Pipeline(BackgroundTask):
 
                 self.parsed = True
 
-            except gobject.GError, e:
+            except GObject.GError as e:
                 show_error('GStreamer error when creating pipeline', str(e))
                 self.error = str(e)
                 self.eos = True
@@ -282,49 +276,61 @@ class Pipeline(BackgroundTask):
                 return
 
             bus.add_signal_watch()
-            watch_id = bus.connect('message', self.on_message)
-            self.watch_id = watch_id
+            self.watch_id = bus.connect('message', self.on_message)
 
-        self.pipeline.set_state(gst.STATE_PLAYING)
+        self.pipeline.set_state(Gst.State.PLAYING)
 
     def stop_pipeline(self):
         if not self.pipeline:
             debug('pipeline already stopped!')
             return
+        self.pipeline.set_state(Gst.State.NULL)
         bus = self.pipeline.get_bus()
         bus.disconnect(self.watch_id)
         bus.remove_signal_watch()
-        self.pipeline.set_state(gst.STATE_NULL)
-        #self.pipeline = None
+        self.pipeline = None
 
     def get_position(self):
         return NotImplementedError
-
+        
+    def query_duration(self):
+        """
+        Ask for the duration of the current pipeline.
+        """
+        try:
+            if not self.sound_file.duration and self.pipeline:
+                self.sound_file.duration = self.pipeline.query_duration(Gst.Format.TIME)[1] / Gst.SECOND
+                if self.sound_file.duration <= 0:
+                    self.sound_file.duration = None
+        except Gst.QueryError:
+            self.sound_file.duration = None
 
 class TypeFinder(Pipeline):
-
     def __init__(self, sound_file):
         Pipeline.__init__(self)
         self.sound_file = sound_file
 
-        command = '%s location="%s" ! typefind name=typefinder ! fakesink' % \
-            (gstreamer_source, encode_filename(self.sound_file.uri))
+        command = '%s location="%s" ! decodebin name=decoder ! fakesink' % \
+                  (gstreamer_source, encode_filename(self.sound_file.uri))
         self.add_command(command)
-        self.add_signal('typefinder', 'have-type', self.have_type)
-
-    def on_error(self, error):
-        self.error = error
-        log('error: %s (%s)' % (error, self.sound_file.filename_for_display))
+        self.add_signal('decoder', 'pad-added', self.pad_added)
+        # 'typefind' is the name of the typefind element created inside
+        # decodebin. we can't use our own typefind before decodebin anymore,
+        # since its caps would've been the same as decodebin's sink caps.
+        self.add_signal('typefind', 'have-type', self.have_type)
 
     def set_found_type_hook(self, found_type_hook):
         self.found_type_hook = found_type_hook
+    
+    def pad_added(self, decoder, pad):
+        """ called when a decoded pad is created """
+        self.query_duration()
+        self.done()
 
     def have_type(self, typefind, probability, caps):
         mime_type = caps.to_string()
-        debug('have_type:', mime_type,
-                                self.sound_file.filename_for_display)
+        debug('have_type:', mime_type, self.sound_file.filename_for_display)
         self.sound_file.mime_type = None
-        #self.sound_file.mime_type = mime_type
         for t in mime_whitelist:
             if t in mime_type:
                 self.sound_file.mime_type = mime_type
@@ -333,24 +339,20 @@ class TypeFinder(Pipeline):
         for t in filename_blacklist:
             if fnmatch(self.sound_file.uri, t):
                 self.sound_file.mime_type = None
-                log('filename blacklisted (%s): %s' % (t,
-                        self.sound_file.filename_for_display))
-                
-        self.pipeline.set_state(gst.STATE_NULL)
-        self.done()
+                log('filename blacklisted (%s): %s' % (t, self.sound_file.filename_for_display))
+        
+        return True
 
     def finished(self):
         Pipeline.finished(self)
         if self.error:
             return
         if self.found_type_hook and self.sound_file.mime_type:
-            gobject.idle_add(self.found_type_hook, self.sound_file,
-                                self.sound_file.mime_type)
-            self.sound_file.mime_type = True # remove string
+            self.found_type_hook(self.sound_file, self.sound_file.mime_type)
+            self.sound_file.mime_type = True
 
 
 class Decoder(Pipeline):
-
     """A GstPipeline background task that decodes data and finds tags."""
 
     def __init__(self, sound_file):
@@ -360,84 +362,75 @@ class Decoder(Pipeline):
         self.position = 0
 
         command = '%s location="%s" name=src ! decodebin name=decoder' % \
-            (gstreamer_source, encode_filename(self.sound_file.uri))
+                  (gstreamer_source, encode_filename(self.sound_file.uri))
         self.add_command(command)
-        self.add_signal('decoder', 'new-decoded-pad', self.new_decoded_pad)
-
-    def on_error(self, error):
-        self.error = error
-        log('error: %s (%s)' % (error,
-            self.sound_file.filename_for_display))
+        self.add_signal('decoder', 'pad-added', self.pad_added)
 
     def have_type(self, typefind, probability, caps):
         pass
 
-    def query_duration(self):
-        """
-        Ask for the duration of the current pipeline.
-        """
-        try:
-            if not self.sound_file.duration and self.pipeline:
-                self.sound_file.duration = self.pipeline.query_duration(
-                                            gst.FORMAT_TIME)[0] / gst.SECOND
-                debug('got file duration:', self.sound_file.duration)
-                if self.sound_file.duration < 0:
-                    self.sound_file.duration = None
-        except gst.QueryError:
-            self.sound_file.duration = None
-
     def query_position(self):
         """
         Ask for the stream position of the current pipeline.
         """
         try:
             if self.pipeline:
-                self.position = self.pipeline.query_position(
-                                            gst.FORMAT_TIME)[0] / gst.SECOND
-                if self.position < 0:
-                    self.position = 0
-        except gst.QueryError:
+                self.position = max(0, self.pipeline.query_position(
+                    Gst.Format.TIME)[1] / Gst.SECOND)
+        except Gst.QueryError:
             self.position = 0
 
-
     def found_tag(self, decoder, something, taglist):
         """
         Called when the decoder reads a tag.
         """
         debug('found_tags:', self.sound_file.filename_for_display)
+        taglist.foreach(self.append_tag, None)
+
+    def append_tag(self, taglist, tag, unused_udata):
         tag_whitelist = (
+            'album-artist',
             'artist',
             'album',
             'title',
             'track-number',
             'track-count',
             'genre',
-            'date',
+            'datetime',
             'year',
             'timestamp',
             'disc-number',
             'disc-count',
         )
+        if tag not in tag_whitelist:
+            return
+
+        tag_type = Gst.tag_get_type(tag)
+        type_getters = {
+            GObject.TYPE_STRING: 'get_string',
+            GObject.TYPE_DOUBLE: 'get_double',
+            GObject.TYPE_FLOAT: 'get_float',
+            GObject.TYPE_INT: 'get_int',
+            GObject.TYPE_UINT: 'get_uint',
+        }
+
         tags = {}
-        for k in taglist.keys():
-            if k in tag_whitelist:
-                tags[k] = taglist[k]
+        if tag_type in type_getters:
+            value = str(getattr(taglist, type_getters[tag_type])(tag)[1])
+            tags[tag] = value
 
-        for k in tags.keys():
-            if 'image' not in k:
-                debug('\t%s=%s (%s)' % (k, tags[k], type(tags[k])))
-            if isinstance(tags[k], gst.Date):
-                tags['year'] = tags[k].year
-                tags['date'] = '%04d-%02d-%02d' % (tags[k].year,
-                                    tags[k].month, tags[k].day)
+        if 'datetime' in tag:
+            dt = taglist.get_date_time(tag)[1]
+            tags['year'] = dt.get_year()
+            tags['date'] = dt.to_iso8601_string()[:10]
 
+        debug('   ', tags)
         self.sound_file.tags.update(tags)
-        self.query_duration()
 
-    def new_decoded_pad(self, decoder, pad, is_last):
+    def pad_added(self, decoder, pad):
         """ called when a decoded pad is created """
-        self.query_duration()
         self.processing = True
+        self.query_duration()
 
     def finished(self):
         Pipeline.finished(self)
@@ -450,7 +443,6 @@ class Decoder(Pipeline):
 
     def get_duration(self):
         """ return the total duration of the sound file """
-        self.query_duration()
         return self.sound_file.duration
 
     def get_position(self):
@@ -460,7 +452,6 @@ class Decoder(Pipeline):
 
 
 class TagReader(Decoder):
-
     """A GstPipeline background task for finding meta tags in a file."""
 
     def __init__(self, sound_file):
@@ -478,7 +469,7 @@ class TagReader(Decoder):
 
     def on_state_changed(self, bus, message):
         prev, new, pending = message.parse_state_changed()
-        if new == gst.STATE_PLAYING and not self.tagread:
+        if new == Gst.State.PLAYING and not self.tagread:
             self.tagread = True
             debug('TagReading done...')
             self.done()
@@ -487,16 +478,15 @@ class TagReader(Decoder):
         Pipeline.finished(self)
         self.sound_file.tags_read = True
         if self.found_tag_hook:
-            gobject.idle_add(self.found_tag_hook, self)
+            GObject.idle_add(self.found_tag_hook, self)
 
 
 class Converter(Decoder):
-
     """A background task for converting files to another format."""
 
     def __init__(self, sound_file, output_filename, output_type,
-                    delete_original=False, output_resample=False,
-                    resample_rate=48000, force_mono=False):
+                 delete_original=False, output_resample=False,
+                 resample_rate=48000, force_mono=False):
         Decoder.__init__(self, sound_file)
 
         self.output_filename = output_filename
@@ -513,6 +503,7 @@ class Converter(Decoder):
         self.resample_rate = resample_rate
         self.force_mono = force_mono
 
+        self.overwrite = False
         self.delete_original = delete_original
 
         self.got_duration = False
@@ -527,45 +518,41 @@ class Converter(Decoder):
             'audio/ogg; codecs=opus': self.add_opus_encoder,
             'gst-profile': self.add_audio_profile,
         }
-        self.add_command('audiorate tolerance=10000000')
+        self.add_command('audiorate')
         self.add_command('audioconvert')
         self.add_command('audioresample')
 
         # audio resampling support
         if self.output_resample:
-            self.add_command('audio/x-raw-int,rate=%d' % self.resample_rate)
+            self.add_command('audio/x-raw,rate=%d' % self.resample_rate)
             self.add_command('audioconvert')
             self.add_command('audioresample')
 
         if self.force_mono:
-            self.add_command('audio/x-raw-int,channels=1')
+            self.add_command('audio/x-raw,channels=1')
             self.add_command('audioconvert')
 
         encoder = self.encoders[self.output_type]()
-        if not encoder:
-            # TODO: is this used ?
-            # TODO: add proper error management when an encoder cannot be created
-            show_error(_('Error', "Cannot create a decoder for \'%s\' format.") % 
-                        self.output_type)
-            return
-
         self.add_command(encoder)
 
-        uri = gnomevfs.URI(self.output_filename)
-        dirname = uri.parent
-        if dirname and not gnomevfs.exists(dirname):
-            log('Creating folder: \'%s\'' % dirname)
-            if not vfs_makedirs(str(dirname)):
-                show_error('Error', _("Cannot create \'%s\' folder.") % dirname)
+        gfile = Gio.file_parse_name(self.output_filename)
+        dirname = gfile.get_parent()
+        if dirname and not dirname.query_exists(None):
+            log('Creating folder: \'%s\'' % beautify_uri(dirname.get_uri()))
+            if not dirname.make_directory_with_parents():
+                show_error('Error', _("Cannot create \'%s\' folder.") % beautify_uri(dirname))
                 return
 
         self.add_command('%s location="%s"' % (
             gstreamer_sink, encode_filename(self.output_filename)))
+        if self.overwrite and vfs_exists(self.output_filename):
+            log('overwriting \'%s\'' % beautify_uri(self.output_filename))
+            vfs_unlink(self.output_filename)
 
     def aborted(self):
         # remove partial file
         try:
-            gnomevfs.unlink(self.output_filename)
+            vfs_unlink(self.output_filename)
         except:
             log('cannot delete: \'%s\'' % beautify_uri(self.output_filename))
         return
@@ -574,28 +561,19 @@ class Converter(Decoder):
         Pipeline.finished(self)
 
         # Copy file permissions
-        try:
-            info = gnomevfs.get_file_info(self.sound_file.uri,
-                                        gnomevfs.FILE_INFO_FIELDS_PERMISSIONS)
-            gnomevfs.set_file_info(self.output_filename, info,
-                                            gnomevfs.SET_FILE_INFO_PERMISSIONS)
-        except:
-            log('Cannot set permission on \'%s\'' %
-                        gnomevfs.format_uri_for_display(self.output_filename))
+        if not Gio.file_parse_name(self.sound_file.uri).copy_attributes(
+                Gio.file_parse_name(self.output_filename), Gio.FileCopyFlags.NONE, None):
+            log('Cannot set permission on \'%s\'' % beautify_uri(self.output_filename))
 
         if self.delete_original and self.processing and not self.error:
             log('deleting: \'%s\'' % self.sound_file.uri)
-            try:
-                vfs_unlink(self.sound_file.uri)
-            except:
-                log('Cannot remove \'%s\'' %
-                        gnomevfs.format_uri_for_display(self.output_filename))
+            if not vfs_unlink(self.sound_file.uri):
+                log('Cannot remove \'%s\'' % beautify_uri(self.output_filename))
 
-    def on_error(self, err):
-        #pass
-        self.error = err
-        show_error('<b>%s</b>' % _('GStreamer Error:'), '%s\n<i>(%s)</i>' % (err,
-                    self.sound_file.filename_for_display))
+    def on_error(self, error):
+        Pipeline.on_error(self, error)
+        show_error('<b>%s</b>' % _('GStreamer Error:'),
+                   '%s\n<i>(%s)</i>' % (error, self.sound_file.filename_for_display))
 
     def set_vorbis_quality(self, quality):
         self.vorbis_quality = quality
@@ -626,8 +604,9 @@ class Converter(Decoder):
         return s
 
     def add_wav_encoder(self):
-        return 'audio/x-raw-int,width=%d ! wavenc' % (
-                self.wav_sample_width)
+        formats = {8:'U8', 16:'S16LE', 24:'S24LE', 32:'S32LE'}
+        return 'audioconvert ! audio/x-raw,format=%s ! wavenc' % (
+            formats[self.wav_sample_width])
 
     def add_oggvorbis_encoder(self):
         cmd = 'vorbisenc'
@@ -641,9 +620,9 @@ class Converter(Decoder):
 
         if self.mp3_mode is not None:
             properties = {
-                'cbr' : 'target=bitrate cbr=true bitrate=%s ',
-                'abr' : 'target=bitrate cbr=false bitrate=%s ',
-                'vbr' : 'target=quality cbr=false quality=%s ',
+                'cbr': 'target=bitrate cbr=true bitrate=%s ',
+                'abr': 'target=bitrate cbr=false bitrate=%s ',
+                'vbr': 'target=quality cbr=false quality=%s ',
             }
 
             cmd += properties[self.mp3_mode] % self.mp3_quality
@@ -652,9 +631,9 @@ class Converter(Decoder):
                 # add xing header when creating VBR/ABR mp3
                 cmd += '! xingmux '
 
-        if 'id3v2mux' in available_elements:
+        if 'id3mux' in available_elements:
             # add tags
-            cmd += '! id3v2mux '
+            cmd += '! id3mux '
 
         return cmd
 
@@ -662,7 +641,7 @@ class Converter(Decoder):
         return 'faac bitrate=%s ! mp4mux' % (self.aac_quality * 1000)
 
     def add_opus_encoder(self):
-        return 'opusenc bitrate=%s ! oggmux' % (self.opus_quality * 1000)
+        return 'opusenc bitrate=%s bitrate-type=vbr bandwidth=auto ! oggmux' % (self.opus_quality * 1000)
 
     def add_audio_profile(self):
         pipeline = audio_profiles_dict[self.audio_profile][2]
@@ -670,20 +649,22 @@ class Converter(Decoder):
 
 
 class ConverterQueue(TaskQueue):
-
     """Background task for converting many files."""
 
     def __init__(self, window):
         TaskQueue.__init__(self)
         self.window = window
+        self.overwrite_action = None
         self.reset_counters()
 
     def reset_counters(self):
-        self.total_duration = 0
         self.duration_processed = 0
+        self.overwrite_action = None
         self.errors = []
         self.error_count = 0
         self.all_tasks = None
+        global user_canceled_codec_installation
+        user_canceled_codec_installation = True
 
     def add(self, sound_file):
         # generate a temporary filename from source name and output suffix
@@ -692,29 +673,32 @@ class ConverterQueue(TaskQueue):
         if vfs_exists(output_filename):
             # always overwrite temporary files
             vfs_unlink(output_filename)
-        
+
+        path = urlparse(output_filename)[2]
+        path = unquote_filename(path)
+
         c = Converter(sound_file, output_filename,
-                        self.window.prefs.get_string('output-mime-type'),
-                        self.window.prefs.get_int('delete-original'),
-                        self.window.prefs.get_int('output-resample'),
-                        self.window.prefs.get_int('resample-rate'),
-                        self.window.prefs.get_int('force-mono'),
-                        )
-        c.set_vorbis_quality(self.window.prefs.get_float('vorbis-quality'))
-        c.set_aac_quality(self.window.prefs.get_int('aac-quality'))
-        c.set_opus_quality(self.window.prefs.get_int('opus-bitrate'))
-        c.set_flac_compression(self.window.prefs.get_int('flac-compression'))
-        c.set_wav_sample_width(self.window.prefs.get_int('wav-sample-width'))
-        c.set_audio_profile(self.window.prefs.get_string('audio-profile'))
+                      self.window.prefs.settings.get_string('output-mime-type'),
+                      self.window.prefs.settings.get_boolean('delete-original'),
+                      self.window.prefs.settings.get_boolean('output-resample'),
+                      self.window.prefs.settings.get_int('resample-rate'),
+                      self.window.prefs.settings.get_boolean('force-mono'),
+        )
+        c.set_vorbis_quality(self.window.prefs.settings.get_double('vorbis-quality'))
+        c.set_aac_quality(self.window.prefs.settings.get_int('aac-quality'))
+        c.set_opus_quality(self.window.prefs.settings.get_int('opus-bitrate'))
+        c.set_flac_compression(self.window.prefs.settings.get_int('flac-compression'))
+        c.set_wav_sample_width(self.window.prefs.settings.get_int('wav-sample-width'))
+        c.set_audio_profile(self.window.prefs.settings.get_string('audio-profile'))
 
         quality = {
             'cbr': 'mp3-cbr-quality',
             'abr': 'mp3-abr-quality',
             'vbr': 'mp3-vbr-quality'
         }
-        mode = self.window.prefs.get_string('mp3-mode')
+        mode = self.window.prefs.settings.get_string('mp3-mode')
         c.set_mp3_mode(mode)
-        c.set_mp3_quality(self.window.prefs.get_int(quality[mode]))
+        c.set_mp3_quality(self.window.prefs.settings.get_int(quality[mode]))
         c.init()
         c.add_listener('finished', self.on_task_finished)
         self.add_task(c)
@@ -731,30 +715,30 @@ class ConverterQueue(TaskQueue):
         for task in self.all_tasks:
             if task.sound_file.duration is None:
                 duration = task.get_duration()
-                if duration:
-                    self.total_duration += duration
 
         position = 0.0
-        prolist = []
-        for task in range(self.finished_tasks): # TODO: use the add, luke
-            prolist.append(1.0)
+        prolist = [1] * self.finished_tasks
         for task in tasks:
             if task.running:
-                position += task.get_position()
-                taskprogress = float(task.get_position()) / task.sound_file.duration if task.sound_file.duration else 0
+                task_position = task.get_position()
+                position += task_position
+                per_file_progress[task.sound_file] = None
+                if task.sound_file.duration is None:
+                    continue
+                taskprogress = task_position / task.sound_file.duration
                 taskprogress = min(max(taskprogress, 0.0), 1.0)
                 prolist.append(taskprogress)
                 per_file_progress[task.sound_file] = taskprogress
         for task in self.waiting_tasks:
             prolist.append(0.0)
 
-        progress = sum(prolist)/len(prolist) if prolist else 0
+        progress = sum(prolist) / len(prolist) if prolist else 0
         progress = min(max(progress, 0.0), 1.0)
         return self.running or len(self.all_tasks), progress
 
     def on_task_finished(self, task):
         task.sound_file.progress = 1.0
-        
+
         if task.error:
             debug('error in task, skipping rename:', task.output_filename)
             if vfs_exists(task.output_filename):
@@ -762,24 +746,24 @@ class ConverterQueue(TaskQueue):
             self.errors.append(task.error)
             self.error_count += 1
             return
-            
+
         duration = task.get_duration()
         if duration:
             self.duration_processed += duration
 
         # rename temporary file 
         newname = self.window.prefs.generate_filename(task.sound_file)
-        log(beautify_uri(task.output_filename), '->', beautify_uri(newname))
-        
+        debug(beautify_uri(task.output_filename), '->', beautify_uri(newname))
+
         # safe mode. generate a filename until we find a free one
-        p,e = os.path.splitext(newname)
+        p, e = os.path.splitext(newname)
         p = p.replace('%', '%%')
         p = p + ' (%d)' + e
         i = 1
         while vfs_exists(newname):
             newname = p % i
             i += 1
-        
+
         task.error = vfs_rename(task.output_filename, newname)
         if task.error:
             self.errors.append(task.error)
@@ -820,3 +804,7 @@ class ConverterQueue(TaskQueue):
         TaskQueue.abort(self)
         self.window.set_sensitive()
         self.reset_counters()
+    
+    def start(self):
+        #self.waiting_tasks.sort(key=Converter.get_duration,reverse=True)
+        TaskQueue.start(self)
